@@ -46,7 +46,6 @@
 #include "TaskContext.hpp"
 #include "internal/CatchConfig.hpp"
 #include "extras/SlaveActivity.hpp"
-#include "os/traces.h"
 
 #include <boost/bind.hpp>
 #include <algorithm>
@@ -70,7 +69,8 @@ namespace RTT
         : taskc(owner),
           mqueue(new MWSRQueue<DisposableInterface*>(ORONUM_EE_MQUEUE_SIZE) ),
           port_queue(new MWSRQueue<PortInterface*>(ORONUM_EE_MQUEUE_SIZE) ),
-          f_queue( new MWSRQueue<ExecutableInterface*>(ORONUM_EE_MQUEUE_SIZE) )
+          f_queue( new MWSRQueue<ExecutableInterface*>(ORONUM_EE_MQUEUE_SIZE) ),
+          mmaster(0)
     {
     }
 
@@ -245,6 +245,11 @@ namespace RTT
         if (taskc && taskc->mTaskState == TaskCore::FatalError )
             return false;
 
+        // forward message to master ExecutionEngine if available
+        if (mmaster) {
+            return mmaster->process(c);
+        }
+
         if ( c && this->getActivity() ) {
             bool result = mqueue->enqueue( c );
             this->getActivity()->trigger();
@@ -260,6 +265,11 @@ namespace RTT
         if (taskc && taskc->mTaskState == TaskCore::FatalError )
             return false;
 
+        // forward port callback to the master ExecutionEngine if available
+        if (mmaster) {
+            return mmaster->process(port);
+        }
+
         if ( port && this->getActivity() ) {
             bool result = port_queue->enqueue( port );
             this->getActivity()->trigger();
@@ -270,12 +280,43 @@ namespace RTT
 
     void ExecutionEngine::waitForMessages(const boost::function<bool(void)>& pred)
     {
+        // forward the call to the master ExecutionEngine which is processing messages for us...
+        if (mmaster) {
+            mmaster->waitForMessages(pred);
+            return;
+        }
+
         if (isSelf())
             waitAndProcessMessages(pred);
         else
             waitForMessagesInternal(pred);
     }
 
+    void ExecutionEngine::setMaster(ExecutionEngine *master)
+    {
+        mmaster = master;
+    }
+
+    void ExecutionEngine::setActivity( base::ActivityInterface* task )
+    {
+        extras::SlaveActivity *slave_activity = dynamic_cast<extras::SlaveActivity *>(task);
+        if (slave_activity && slave_activity->getMaster()) {
+            ExecutionEngine *master = dynamic_cast<ExecutionEngine *>(slave_activity->getMaster()->getRunner());
+            setMaster(master);
+        } else {
+            setMaster(0);
+        }
+        RTT::base::RunnableInterface::setActivity(task);
+    }
+
+    os::ThreadInterface* ExecutionEngine::getThread() const {
+        // forward to the master ExecutionEngine if available
+        if (mmaster) {
+            return mmaster->getThread();
+        }
+        return base::RunnableInterface::getThread();
+    }
+
     bool ExecutionEngine::isSelf() const {
         os::ThreadInterface *thread = this->getThread();
         return (thread && thread->isSelf());
@@ -283,6 +324,7 @@ namespace RTT
 
     void ExecutionEngine::waitForMessagesInternal(boost::function<bool(void)> const& pred)
     {
+        assert( mmaster == 0 );
         if ( pred() )
             return;
         // only to be called from the thread not executing step().
@@ -295,6 +337,7 @@ namespace RTT
 
     void ExecutionEngine::waitAndProcessMessages(boost::function<bool(void)> const& pred)
     {
+        assert( mmaster == 0 );
         // optimization for the case the predicate is already true
         if ( pred() )
             return;
@@ -355,8 +398,7 @@ namespace RTT
             // A trigger() in startHook() will be ignored, we trigger in TaskCore after startHook finishes.
             if ( taskc->mTaskState == TaskCore::Running && taskc->mTargetState == TaskCore::Running ) {
                 TRY (
-                    { tracepoint_context(orocos_rtt, TaskContext_updateHook, taskc->mName.c_str());
-                        taskc->updateHook(); }
+                    taskc->updateHook();
                 ) CATCH(std::exception const& e,
                     log(Error) << "in updateHook(): switching to exception state because of unhandled exception" << endlog();
                     log(Error) << "  " << e.what() << endlog();
@@ -369,8 +411,7 @@ namespace RTT
             // in case start() or updateHook() called error(), this will be called:
             if (taskc->mTaskState == TaskCore::RunTimeError && taskc->mTargetState >= TaskCore::Running) {
                 TRY (
-                    { tracepoint_context(orocos_rtt, TaskContext_errorHook, taskc->mName.c_str());
-                        taskc->errorHook(); }
+                    taskc->errorHook();
                 ) CATCH(std::exception const& e,
                     log(Error) << "in errorHook(): switching to exception state because of unhandled exception" << endlog();
                     log(Error) << "  " << e.what() << endlog();
