@@ -42,15 +42,13 @@
 #include "ActivityInterface.hpp"
 #include "Logger.hpp"
 #include "internal/CatchConfig.hpp"
-#include <rtt/os/traces.h>
-#include <cstring>
 
 namespace RTT {
     using namespace detail;
 
     using namespace std;
 
-    TaskCore::TaskCore(TaskState initial_state /*= Stopped*/, const std::string& name /* = std::string() */  )
+    TaskCore::TaskCore(TaskState initial_state /*= Stopped*/ )
         :  ee( new ExecutionEngine(this) )
            ,mTaskState(initial_state)
            ,mInitialState(initial_state)
@@ -60,7 +58,6 @@ namespace RTT {
            ,mIOCounter(0)
            ,mTimeOutCounter(0)
            ,mTriggerCounter(0)
-           ,mName(name)
     {
     }
 
@@ -97,9 +94,7 @@ namespace RTT {
         if ( mTaskState == Stopped || mTaskState == PreOperational) {
             TRY(
                 mTargetState = Stopped;
-                bool successful;
-                { tracepoint_context(orocos_rtt, TaskContext_configureHook, mName.c_str());
-                    successful = configureHook(); }
+                bool successful = configureHook();
                 if (successful) {
                     if (mTaskState != Stopped && (mTaskState == mTargetState)) {
                         log(Error) << "in configure(): state has been changed inside the configureHook" << endlog();
@@ -131,8 +126,7 @@ namespace RTT {
         if ( mTaskState == Stopped ) {
             TRY(
                 mTargetState = PreOperational;
-                { tracepoint_context(orocos_rtt, TaskContext_cleanupHook, mName.c_str());
-                    cleanupHook(); }
+                cleanupHook();
                 if (mTaskState == Stopped)
                     mTaskState = PreOperational;
                 return true;
@@ -166,12 +160,10 @@ namespace RTT {
         mTargetState = mTaskState = Exception;
         TRY (
             if ( copy >= Running ) {
-                { tracepoint_context(orocos_rtt, TaskContext_stopHook, mName.c_str());
-                    stopHook(); }
+                stopHook();
             }
             if ( copy >= Stopped && mInitialState == PreOperational ) {
-                { tracepoint_context(orocos_rtt, TaskContext_cleanupHook, mName.c_str());
-                    cleanupHook(); }
+                cleanupHook();
             }
             exceptionHook();
         ) CATCH(std::exception const& e,
@@ -199,9 +191,7 @@ namespace RTT {
         if ( mTaskState == Stopped ) {
             TRY (
                 mTargetState = Running;
-                bool successful;
-                { tracepoint_context(orocos_rtt, TaskContext_startHook, mName.c_str());
-                    successful = startHook(); }
+                bool successful = startHook();
                 if (successful) {
                     if (mTaskState != Running && (mTargetState == mTaskState)) {
                         log(Error) << "in start(): state has been changed inside the startHook" << endlog();
@@ -235,8 +225,7 @@ namespace RTT {
             TRY(
                 mTargetState = Stopped;
                 if ( engine()->stopTask(this) ) {
-                    { tracepoint_context(orocos_rtt, TaskContext_stopHook, mName.c_str());
-                        stopHook(); }
+                    stopHook();
                     mTaskState = Stopped;
                     return true;
                 } else {
@@ -336,3 +325,4 @@ namespace RTT {
     {
     }
 }
+
