@@ -142,8 +142,8 @@ PortableServer::POA_ptr CDataFlowInterface_i::_default_POA()
 }
 
 CDataFlowInterface::CPortNames * CDataFlowInterface_i::getPorts() ACE_THROW_SPEC ((
-          CORBA::SystemException
-        ))
+	      CORBA::SystemException
+	    ))
 {
     ::RTT::DataFlowInterface::PortNames ports = mdf->getPortNames();
 
@@ -157,8 +157,8 @@ CDataFlowInterface::CPortNames * CDataFlowInterface_i::getPorts() ACE_THROW_SPEC
 }
 
 CDataFlowInterface::CPortDescriptions* CDataFlowInterface_i::getPortDescriptions() ACE_THROW_SPEC ((
-          CORBA::SystemException
-        ))
+	      CORBA::SystemException
+	    ))
 {
     RTT::DataFlowInterface::PortNames ports = mdf->getPortNames();
     RTT::corba::CDataFlowInterface::CPortDescriptions_var result = new RTT::corba::CDataFlowInterface::CPortDescriptions();
@@ -171,7 +171,6 @@ CDataFlowInterface::CPortDescriptions* CDataFlowInterface_i::getPortDescriptions
 
         PortInterface* port = mdf->getPort(ports[i]);
         port_desc.name = CORBA::string_dup(ports[i].c_str());
-        port_desc.description = CORBA::string_dup(port->getDescription().c_str());
 
         TypeInfo const* type_info = port->getTypeInfo();
         if (!type_info || !type_info->getProtocol(ORO_CORBA_PROTOCOL_ID))
@@ -193,9 +192,9 @@ CDataFlowInterface::CPortDescriptions* CDataFlowInterface_i::getPortDescriptions
 }
 
 CPortType CDataFlowInterface_i::getPortType(const char * port_name) ACE_THROW_SPEC ((
-          CORBA::SystemException
-          ,::RTT::corba::CNoSuchPortException
-        ))
+	      CORBA::SystemException
+	      ,::RTT::corba::CNoSuchPortException
+	    ))
 {
     PortInterface* p = mdf->getPort(port_name);
     if (p == 0)
@@ -207,9 +206,9 @@ CPortType CDataFlowInterface_i::getPortType(const char * port_name) ACE_THROW_SP
 }
 
 char* CDataFlowInterface_i::getDataType(const char * port_name) ACE_THROW_SPEC ((
-          CORBA::SystemException
-          ,::RTT::corba::CNoSuchPortException
-        ))
+	      CORBA::SystemException
+	      ,::RTT::corba::CNoSuchPortException
+	    ))
 {
     PortInterface* p = mdf->getPort(port_name);
     if ( p == 0)
@@ -218,9 +217,9 @@ char* CDataFlowInterface_i::getDataType(const char * port_name) ACE_THROW_SPEC (
 }
 
 CORBA::Boolean CDataFlowInterface_i::isConnected(const char * port_name) ACE_THROW_SPEC ((
-          CORBA::SystemException
-          ,::RTT::corba::CNoSuchPortException
-        ))
+	      CORBA::SystemException
+	      ,::RTT::corba::CNoSuchPortException
+	    ))
 {
     PortInterface* p = mdf->getPort(port_name);
     if (p == 0)
@@ -240,10 +239,47 @@ void CDataFlowInterface_i::deregisterChannel(CChannelElement_ptr channel)
     }
 }
 
+CORBA::Boolean CDataFlowInterface_i::channelReady(const char * reader_port_name, CChannelElement_ptr channel, CConnPolicy const& policy ) ACE_THROW_SPEC ((
+	      CORBA::SystemException
+	      ,::RTT::corba::CNoSuchPortException
+	    ))
+{
+    PortInterface* p = mdf->getPort(reader_port_name);
+    if (p == 0)
+        throw corba::CNoSuchPortException();
+
+    InputPortInterface* ip = dynamic_cast<InputPortInterface*>(p);
+    if (ip == 0)
+        throw corba::CNoSuchPortException();
+
+    CORBA_CHECK_THREAD();
+    // lookup the C++ channel that matches the corba channel and
+    // inform our local port that that C++ channel is ready.
+    { RTT::os::MutexLock lock(channel_list_mtx);
+        ChannelList::iterator it=channel_list.begin();
+        for (; it != channel_list.end(); ++it) {
+            if (it->first->_is_equivalent (channel) ) {
+                try {
+                    ConnPolicy cp;
+                    cp=toRTT(policy);
+                    return ip->channelReady( it->second, cp );
+                }
+                catch(std::exception const& e)
+                {
+                    log(Error) << "call to channelReady threw " << e.what() << endlog();
+                    throw;
+                }
+            }
+        }
+    }
+    log(Error) << "Invalid CORBA channel given for port " << reader_port_name << ": could not match it to a local C++ channel." <<endlog();
+    return false;
+}
+
 void CDataFlowInterface_i::disconnectPort(const char * port_name) ACE_THROW_SPEC ((
-          CORBA::SystemException
-          ,::RTT::corba::CNoSuchPortException
-        ))
+	      CORBA::SystemException
+	      ,::RTT::corba::CNoSuchPortException
+	    ))
 {
     PortInterface* p = mdf->getPort(port_name);
     if (p == 0) {
@@ -257,9 +293,9 @@ void CDataFlowInterface_i::disconnectPort(const char * port_name) ACE_THROW_SPEC
 bool CDataFlowInterface_i::removeConnection(
         const char* local_port,
         CDataFlowInterface_ptr remote_interface, const char* remote_port) ACE_THROW_SPEC ((
-          CORBA::SystemException
-          ,::RTT::corba::CNoSuchPortException
-        ))
+        	      CORBA::SystemException
+        	      ,::RTT::corba::CNoSuchPortException
+        	    ))
 {
     PortInterface* port = mdf->getPort(local_port);
     // CORBA does not support disconnecting from the input port
@@ -294,9 +330,9 @@ bool CDataFlowInterface_i::removeConnection(
 
 ::CORBA::Boolean CDataFlowInterface_i::createStream( const char* port,
                                RTT::corba::CConnPolicy & policy) ACE_THROW_SPEC ((
-          CORBA::SystemException
-          ,::RTT::corba::CNoSuchPortException
-        ))
+                               	      CORBA::SystemException
+                               	      ,::RTT::corba::CNoSuchPortException
+                               	    ))
 {
     PortInterface* p = mdf->getPort(port);
     if (p == 0) {
@@ -314,9 +350,9 @@ bool CDataFlowInterface_i::removeConnection(
 }
 
 void CDataFlowInterface_i::removeStream( const char* port, const char* stream_name) ACE_THROW_SPEC ((
-          CORBA::SystemException
-          ,::RTT::corba::CNoSuchPortException
-        ))
+	      CORBA::SystemException
+	      ,::RTT::corba::CNoSuchPortException
+	    ))
 {
     PortInterface* p = mdf->getPort(port);
     if (p == 0) {
@@ -331,11 +367,10 @@ void CDataFlowInterface_i::removeStream( const char* port, const char* stream_na
 
 CChannelElement_ptr CDataFlowInterface_i::buildChannelOutput(
         const char* port_name, CConnPolicy & corba_policy) ACE_THROW_SPEC ((
-          CORBA::SystemException
-          ,::RTT::corba::CNoCorbaTransport
-          ,::RTT::corba::CNoSuchPortException
-          ,::RTT::corba::CInvalidArgument
-        ))
+         	      CORBA::SystemException
+         	      ,::RTT::corba::CNoCorbaTransport
+                  ,::RTT::corba::CNoSuchPortException
+         	    ))
 {
     Logger::In in("CDataFlowInterface_i::buildChannelOutput");
     InputPortInterface* port = dynamic_cast<InputPortInterface*>(mdf->getPort(port_name));
@@ -355,36 +390,9 @@ CChannelElement_ptr CDataFlowInterface_i::buildChannelOutput(
     // Convert to RTT policy.
     ConnPolicy policy2 = toRTT(corba_policy);
 
-    // For shared push connections, also build or check the local shared connection instance here
-    ChannelElementBase::shared_ptr end;
-    if (policy2.buffer_policy == Shared) {
-        internal::SharedConnectionBase::shared_ptr shared_connection = type_info->buildSharedConnection(0, port, policy2);
-        if (!shared_connection) {
-            throw CInvalidArgument();
-        }
-
-        // If no user supplied name, pass on the new name.
-        if ( strlen( corba_policy.name_id.in()) == 0 )
-            corba_policy.name_id = CORBA::string_dup( shared_connection->getName().c_str() );
-
-        // Connect the input port if it is not already connected
-        if (!port->createConnection(shared_connection, policy2)) {
-            return RTT::corba::CChannelElement::_nil();
-        }
-
-        end = shared_connection;
-
-    // ...otherwise, create default channel output.
-    } else {
-        end = type_info->buildChannelOutput(*port, policy2);
-        if (!end) throw CInvalidArgument();
-    }
-
+    ChannelElementBase::shared_ptr end = type_info->buildChannelOutput(*port);
     CRemoteChannelElement_i* this_element =
-        transporter->createChannelElement_i(mdf, mpoa, policy2);
-    // transporter could be the CorbaFallBackProtocol => createChannelElement_i() returns null pointer
-    if (!this_element)
-        throw CNoCorbaTransport();
+        transporter->createChannelElement_i(mdf, mpoa, corba_policy.pull);
     this_element->setCDataFlowInterface(this);
 
     /*
@@ -398,15 +406,17 @@ CChannelElement_ptr CDataFlowInterface_i::buildChannelOutput(
             log(Error) << "No such transport registered. Check your corba_policy.transport settings or add the transport for type "<< type_info->getTypeName() <<endlog();
             return RTT::corba::CChannelElement::_nil();
         }
-        RTT::base::ChannelElementBase::shared_ptr ceb = type_info->getProtocol(corba_policy.transport)->createStream(port, policy2, /* is_sender = */ false);
+        RTT::base::ChannelElementBase::shared_ptr ceb = type_info->getProtocol(corba_policy.transport)->createStream(port, policy2, false);
         // if no user supplied name, pass on the new name.
         if ( strlen( corba_policy.name_id.in()) == 0 )
             corba_policy.name_id = CORBA::string_dup( policy2.name_id.c_str() );
 
         if (ceb) {
             // override, insert oob element between corba and endpoint and add a buffer between oob and endpoint.
-            dynamic_cast<ChannelElementBase*>(this_element)->connectTo(ceb, policy2.mandatory);
-            ceb->getOutputEndPoint()->connectTo(end, policy2.mandatory);
+            dynamic_cast<ChannelElementBase*>(this_element)->setOutput(ceb);
+            ChannelElementBase::shared_ptr buf = type_info->buildDataStorage(toRTT(corba_policy));
+            ceb->setOutput( buf );
+            buf->setOutput(end);
             log(Info) <<"Receiving data for port "<< policy2.name_id << " from out-of-band protocol "<< corba_policy.transport <<endlog();
         } else {
             log(Error) << "The type transporter for type "<<type_info->getTypeName()<< " failed to create an out-of-band endpoint for port " << port_name<<endlog();
@@ -414,15 +424,21 @@ CChannelElement_ptr CDataFlowInterface_i::buildChannelOutput(
         }
         //
     } else {
-        // No OOB.
-        dynamic_cast<ChannelElementBase*>(this_element)->connectTo(end, policy2.mandatory);
+        // No OOB. omit buffer if in pull
+        if ( !corba_policy.pull ) {
+            ChannelElementBase::shared_ptr buf = type_info->buildDataStorage(toRTT(corba_policy));
+            dynamic_cast<ChannelElementBase*>(this_element)->setOutput(buf);
+            buf->setOutput(end);
+        } else {
+            dynamic_cast<ChannelElementBase*>(this_element)->setOutput(end);
+        }
     }
 
     this_element->_remove_ref();
 
     // store our mapping of corba channel elements to C++ channel elements. We need this for channelReady() and removing a channel again.
     { RTT::os::MutexLock lock(channel_list_mtx);
-        channel_list.push_back( ChannelList::value_type(this_element->_this(), end) );
+        channel_list.push_back( ChannelList::value_type(this_element->_this(), end->getOutputEndPoint()));
     }
 
     CRemoteChannelElement_var proxy = this_element->_this();
@@ -434,11 +450,10 @@ CChannelElement_ptr CDataFlowInterface_i::buildChannelOutput(
  */
 CChannelElement_ptr CDataFlowInterface_i::buildChannelInput(
         const char* port_name, CConnPolicy & corba_policy) ACE_THROW_SPEC ((
-          CORBA::SystemException
-          ,::RTT::corba::CNoCorbaTransport
-          ,::RTT::corba::CNoSuchPortException
-          ,::RTT::corba::CInvalidArgument
-        ))
+        	      CORBA::SystemException
+        	      ,::RTT::corba::CNoCorbaTransport
+        	      ,::RTT::corba::CNoSuchPortException
+        	    ))
 {
     Logger::In in("CDataFlowInterface_i::buildChannelInput");
     // First check validity of user input...
@@ -460,19 +475,16 @@ CChannelElement_ptr CDataFlowInterface_i::buildChannelInput(
     ConnPolicy policy2 = toRTT(corba_policy);
 
     // Now create the output-side channel elements.
-    ChannelElementBase::shared_ptr start = type_info->buildChannelInput(*port, policy2);
-    if (!start) {
-        throw CInvalidArgument();
-    }
+    ChannelElementBase::shared_ptr start = type_info->buildChannelInput(*port);
 
     // The channel element that exposes our channel in CORBA
     CRemoteChannelElement_i* this_element;
-    PortableServer::ServantBase_var servant = this_element = transporter->createChannelElement_i(mdf, mpoa, policy2);
-    // transporter could be the CorbaFallBackProtocol => createChannelElement_i() returns null pointer
-    if (!this_element)
-        throw CNoCorbaTransport();
+    PortableServer::ServantBase_var servant = this_element = transporter->createChannelElement_i(mdf, mpoa, corba_policy.pull);
     this_element->setCDataFlowInterface(this);
+
+    // Attach the corba channel element first (so OOB is after corba).
     assert( dynamic_cast<ChannelElementBase*>(this_element) );
+    start->getOutputEndPoint()->setOutput( dynamic_cast<ChannelElementBase*>(this_element));
 
     /*
      * This part if for out-of band. (needs to be factored out).
@@ -485,31 +497,29 @@ CChannelElement_ptr CDataFlowInterface_i::buildChannelInput(
             log(Error) << "No such transport registered. Check your corba_policy.transport settings or add the transport for type "<< type_info->getTypeName() <<endlog();
             throw CNoCorbaTransport();
         }
-        RTT::base::ChannelElementBase::shared_ptr ceb = type_info->getProtocol(corba_policy.transport)->createStream(port, policy2, /* is_sender = */ true);
+        RTT::base::ChannelElementBase::shared_ptr ceb = type_info->getProtocol(corba_policy.transport)->createStream(port, policy2, true);
         // if no user supplied name, pass on the new name.
         if ( strlen( corba_policy.name_id.in()) == 0 )
             corba_policy.name_id = CORBA::string_dup( policy2.name_id.c_str() );
 
         if (ceb) {
             // OOB is added to end of chain.
-            start->connectTo( dynamic_cast<ChannelElementBase*>(this_element), policy2.mandatory );
-            dynamic_cast<ChannelElementBase*>(this_element)->connectTo( ceb );
+            start->getOutputEndPoint()->setOutput( ceb );
             log(Info) <<"Sending data from port "<< policy2.name_id << " to out-of-band protocol "<< corba_policy.transport <<endlog();
         } else {
             log(Error) << "The type transporter for type "<<type_info->getTypeName()<< " failed to create an out-of-band endpoint for port " << port_name<<endlog();
             throw CNoCorbaTransport();
         }
-
     } else {
         // No OOB. Always add output buffer.
         ChannelElementBase::shared_ptr buf = type_info->buildDataStorage(toRTT(corba_policy));
-        start->connectTo(buf, policy2.mandatory);
-        buf->connectTo( dynamic_cast<ChannelElementBase*>(this_element) );
+        start->setOutput(buf);
+        buf->setOutput( dynamic_cast<ChannelElementBase*>(this_element) );
     }
 
 
     // Attach to our output port:
-    port->addConnection( new SimpleConnID(), start->getInputEndPoint(), policy2 );
+    port->addConnection( new SimpleConnID(), start->getInputEndPoint(), policy2);
 
     // DO NOT DO THIS: _remove_ref() is tied to the refcount of the ChannelElementBase !
     //this_element->_remove_ref();
@@ -522,48 +532,13 @@ CChannelElement_ptr CDataFlowInterface_i::buildChannelInput(
     return this_element->_this();
 }
 
-::CORBA::Boolean CDataFlowInterface_i::createSharedConnection(const char* port_name, ::RTT::corba::CConnPolicy& corba_policy) ACE_THROW_SPEC ((
-          CORBA::SystemException
-          ,::RTT::corba::CNoSuchPortException
-          ,::RTT::corba::CInvalidArgument
-        ))
-{
-    Logger::In in("CDataFlowInterface_i::createSharedConnection");
-    InputPortInterface* port = dynamic_cast<InputPortInterface*>(mdf->getPort(port_name));
-    if (port == 0)
-        throw CNoSuchPortException();
-
-    TypeInfo const* type_info = port->getTypeInfo();
-    if (!type_info)
-        throw CNoCorbaTransport();
-
-    if (corba_policy.buffer_policy != CShared) {
-        throw CInvalidArgument();
-    }
-
-    CORBA_CHECK_THREAD();
-
-    // Convert to RTT policy.
-    ConnPolicy policy2 = toRTT(corba_policy);
-
-    // For shared push connections, also build or check the local shared connection instance here
-    internal::SharedConnectionBase::shared_ptr shared_connection;
-    if (!internal::ConnFactory::findSharedConnection(0, port, policy2, shared_connection) || !shared_connection) return false;
-
-    // If no user supplied name, pass on the new name.
-    if ( strlen( corba_policy.name_id.in()) == 0 )
-        corba_policy.name_id = CORBA::string_dup( shared_connection->getName().c_str() );
-
-    // connect the port
-    return port->createConnection(shared_connection, policy2);
-}
 
 ::CORBA::Boolean CDataFlowInterface_i::createConnection(
         const char* writer_port, CDataFlowInterface_ptr reader_interface,
         const char* reader_port, CConnPolicy & policy) ACE_THROW_SPEC ((
-          CORBA::SystemException
-          ,::RTT::corba::CNoSuchPortException
-        ))
+        	      CORBA::SystemException
+        	      ,::RTT::corba::CNoSuchPortException
+        	    ))
 {
     Logger::In in("CDataFlowInterface_i::createConnection");
     OutputPortInterface* writer = dynamic_cast<OutputPortInterface*>(mdf->getPort(writer_port));
@@ -604,10 +579,7 @@ CChannelElement_ptr CDataFlowInterface_i::buildChannelInput(
         RemoteInputPort port(writer->getTypeInfo(), reader_interface, reader_port, mpoa);
         port.setInterface( mdf ); // cheating !
         // Connect to proxy.
-        ConnPolicy policy2 = toRTT(policy);
-        bool result = writer->createConnection(port, policy2);
-        policy = toCORBA(policy2);
-        return result;
+        return writer->createConnection(port, toRTT(policy));
     }
     catch(CORBA::COMM_FAILURE&) { throw; }
     catch(CORBA::TRANSIENT&) { throw; }
@@ -617,21 +589,16 @@ CChannelElement_ptr CDataFlowInterface_i::buildChannelInput(
 
 // standard constructor
 CRemoteChannelElement_i::CRemoteChannelElement_i(RTT::corba::CorbaTypeTransporter const& transport,
-    PortableServer::POA_ptr poa)
+	  PortableServer::POA_ptr poa)
     : transport(transport)
     , mpoa(PortableServer::POA::_duplicate(poa))
     , mdataflow(0)
-{}
+    { }
 CRemoteChannelElement_i::~CRemoteChannelElement_i() {}
-
 PortableServer::POA_ptr CRemoteChannelElement_i::_default_POA()
-{
-    return PortableServer::POA::_duplicate(mpoa);
-}
-
+{ return PortableServer::POA::_duplicate(mpoa); }
 void CRemoteChannelElement_i::setRemoteSide(CRemoteChannelElement_ptr remote) ACE_THROW_SPEC ((
-          CORBA::SystemException
-        ))
-{
-    this->remote_side = RTT::corba::CRemoteChannelElement::_duplicate(remote);
-}
+	      CORBA::SystemException
+	    ))
+{ this->remote_side = RTT::corba::CRemoteChannelElement::_duplicate(remote); }
+
